package parkomat.model;

import java.time.LocalTime;
import java.util.Comparator;

public class FasciaTariffaria implements Comparable<FasciaTariffaria> {
	
	private LocalTime oraDa, oraFinoA;
	private PagamentoAttivo siPaga;

	public static final FasciaTariffaria GIORNATA_GRATUITA = new FasciaTariffaria(LocalTime.of(0,0), LocalTime.of(0,0), PagamentoAttivo.GRATUITO);			
	
	public FasciaTariffaria(LocalTime daOra, LocalTime aOra, PagamentoAttivo siPaga) {
		this.oraDa = daOra;
		this.oraFinoA = aOra;
		this.siPaga=siPaga;
	}

	public LocalTime getOraDa() {
		return oraDa;
	}

	public LocalTime getOraFinoA() {
		return oraFinoA;
	}
	
	public PagamentoAttivo getPagamentoAttivo() {
		return siPaga;
	}

	public boolean isPagamentoAttivo() {
		return siPaga==PagamentoAttivo.A_PAGAMENTO;
	}

	public boolean contains(LocalTime h) {
		return 	(this.getOraDa().equals(h) || this.getOraDa().isBefore(h))	&&
				(this.getOraFinoA().minusMinutes(1).equals(h) || this.getOraFinoA().minusMinutes(1).isAfter(h));
	}

	@Override
	public int compareTo(FasciaTariffaria that) {
		Comparator<FasciaTariffaria> c = Comparator.comparing(FasciaTariffaria::getOraDa);
		return c.compare(this, that);
	}
	
	@Override
	public boolean equals(Object that) {
		if (!(that instanceof FasciaTariffaria)) return false;
		Comparator<FasciaTariffaria> c = Comparator.comparing(FasciaTariffaria::getOraDa);
		return c.compare(this, (FasciaTariffaria)that)==0;
	}

	@Override
	public String toString() {
		return "FasciaTariffaria " +
				getPagamentoAttivo() +
				" [oraDa=" + oraDa + ", oraFinoA=" + oraFinoA +"]";
	}

}
