package parkomat.model;

import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;

public class MyTicketIssuer extends TicketIssuer {

	public MyTicketIssuer(Map<String, TariffaValida> mappaTariffe) {
		super(mappaTariffe);
	}

	protected double costoSostaInUnGiorno(Tariffa tariffa, DayOfWeek day, LocalTime oraDa, LocalTime oraA) {
		if (!tariffa.isValid(day)) throw new IllegalArgumentException("Tariffa non valida il " + day);
		// algoritmo di calcolo standard
		// 1) ricerca fasce corrispondente a ora iniziale e finale della sosta
		List<FasciaTariffaria> fasce = tariffa.getFasce(day);
		int indexFasciaIniziale = cercaFascia(fasce, oraDa);
		// NB: ai fini della ricerca fascia, l'ultimo istante della giornata sono le 23:59, non le 0:00 del giorno seguente
		// Quindi occorre sottrarre 1 minuto all'orario di fine sosta "oraA", che invece è aperto a destra
		// ESEMPIO: una sosta "dalle 7:45 alle 8:45", ha come ultimo minuto le 8:44, è quello che determina la fascia finale, non le 8:45
		// INVECE internamente ci fa più comodo lavorare con intervalli chiusi a destra per cercare le fasce --> tolgo 1 minuto
		int indexFasciaFinale   = cercaFascia(fasce, oraA.minusMinutes(1));
		// 2a) sosta entro un'unica fascia: facile!
		if (indexFasciaIniziale==indexFasciaFinale) {
			return fasce.get(indexFasciaIniziale).isPagamentoAttivo() ? calcolaCosto(tariffa.getCostoOrario(), oraDa, oraA) : 0;
		}
		else {
		// 2b) sosta distibuita fra più fasce: algoritmo generale
			FasciaTariffaria fasciaIniziale = fasce.get(indexFasciaIniziale);
			double costo = 0;
			if (fasciaIniziale.isPagamentoAttivo()) {
				costo += calcolaCosto(tariffa.getCostoOrario(), oraDa, fasciaIniziale.getOraFinoA());
			}
			//	System.out.println("costo in fascia iniziale: " + costo);			
			for (int i=indexFasciaIniziale+1; i<indexFasciaFinale; i++){
				FasciaTariffaria fasciaCorrente = fasce.get(i);
				if (fasciaCorrente.isPagamentoAttivo()) {
					double costoInFascia = calcolaCosto(tariffa.getCostoOrario(), fasciaCorrente.getOraDa(), fasciaCorrente.getOraFinoA());
					//	System.out.println("costo in fascia " + i + "-esima: " + costoInFascia);
					costo += costoInFascia; 
				}
			}
			FasciaTariffaria fasciaFinale = fasce.get(indexFasciaFinale);
			double costoInFasciaFinale = calcolaCosto(tariffa.getCostoOrario(), fasciaFinale.getOraDa(), oraA);
			if (fasciaFinale.isPagamentoAttivo()) {
				//	System.out.println("costo in fascia finale: " + costoInFasciaFinale);
				costo += costoInFasciaFinale;
			}
			//	System.out.println("costo complessivo: " + costo);
			return costo;
		}
	}
	
}
