package parkomat.model;

import java.time.DayOfWeek;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;
import java.util.Map;

public abstract class TicketIssuer {
	
	private Map<String,TariffaValida> mappaTariffe;
	
	public TicketIssuer(Map<String, TariffaValida> mappaTariffe) {
		this.mappaTariffe = mappaTariffe;
	}

	public Ticket emitTicket(String nomeTariffa, LocalDateTime inizio, LocalDateTime fine) {
		Tariffa tariffa = mappaTariffe.get(nomeTariffa);
		// franchigia: si pospone l'inizio effettivo della sosta
		LocalDateTime inizioEffettivo = inizio.plusMinutes(tariffa.getFranchigia());
		//	System.out.println("inizio effettivo: " + inizioEffettivo + " (franchigia " + tariffa.getFranchigia()+")");	
		double costo;
		// minimo di durata: se durata effettiva è inferiore, si considera la durata minima
		long durataSosta = Duration.between(inizioEffettivo,fine).toMinutes();
		//	System.out.println("durata sosta: " + durataSosta/60.0 + " ore");
		if (durataSosta<tariffa.getDurataSostaMinima()) 
			costo =  tariffa.getDurataSostaMinima() * tariffa.getCostoOrario() / 60.0;
		else 
			costo = costoSosta(tariffa, inizioEffettivo, fine);
		return new Ticket(inizio, fine, costo); //NB: nel ticket metto orario ufficiale, non effettivo
	}
	
	protected double costoSosta(Tariffa tariffa, LocalDateTime inizioEffettivo, LocalDateTime fine) {
		//	System.out.println("costoSosta: " + inizioEffettivo + " - " + fine);
		if (inizioEffettivo.toLocalDate().equals(fine.toLocalDate())) { 
			// System.out.println("singolo giorno: " + inizioEffettivo.toLocalDate());
			return costoSostaInUnGiorno(tariffa, inizioEffettivo.toLocalDate().getDayOfWeek(), inizioEffettivo.toLocalTime(), fine.toLocalTime());
		}
		else {
			// System.out.println("più giorni: " + inizioEffettivo.toLocalDate() + ", " + fine.toLocalDate());
			return costoSostaSuPiuGiorni(tariffa, inizioEffettivo, fine);
		}
	}

	protected abstract double costoSostaInUnGiorno(Tariffa tariffa, DayOfWeek day, LocalTime oraDa, LocalTime oraA);

	protected double costoSostaSuPiuGiorni(Tariffa tariffa, LocalDateTime inizio, LocalDateTime fine) {
		double costo = costoSostaInUnGiorno(tariffa, inizio.toLocalDate().getDayOfWeek(), inizio.toLocalTime(), LocalTime.of(0,0));
		//	System.out.println("costo il " + inizio.toLocalDate().getDayOfWeek() + "   "+ costo);
		for (LocalDate day = inizio.toLocalDate().plusDays(1); day.isBefore(fine.toLocalDate()); day = day.plusDays(1)) {
			//	System.out.println("giorno: " + day);	
			//	double costoOggi = costoSostaInUnGiorno(tariffa, day.getDayOfWeek(), LocalTime.of(0,0), LocalTime.of(0,0));
			//	System.out.println("costo oggi " + day.getDayOfWeek() + "   "+ costoOggi);
			costo += costoSostaInUnGiorno(tariffa, day.getDayOfWeek(), LocalTime.of(0,0), LocalTime.of(0,0));
		}
		//	System.out.println("giorno: " + fine.toLocalDate());
		//	double costoUltimoGiorno = costoSostaInUnGiorno(tariffa, fine.toLocalDate().getDayOfWeek(), LocalTime.of(0,0), fine.toLocalTime());
		//	System.out.println("costo ultimo " + fine.toLocalDate().getDayOfWeek() + "   "+ costoUltimoGiorno);
		costo += costoSostaInUnGiorno(tariffa, fine.toLocalDate().getDayOfWeek(), LocalTime.of(0,0), fine.toLocalTime());
		return costo;
	}
	
	// -------------- utility methods
	
	protected int cercaFascia(List<FasciaTariffaria> fasce, LocalTime orario) {
		int indexFascia=-1; // fake initialisation
		// System.out.println("searching " + orario); // DEBUG
		for(int i=0; i<fasce.size(); i++) {
			FasciaTariffaria f = fasce.get(i);
			//System.out.println(" in " + f + ": " + f.contains(orario)); // DEBUG
			if (f.contains(orario)) indexFascia = i; 
		}
		return indexFascia;
	}

	protected double calcolaCosto(double costoOrario, LocalTime da, LocalTime a) {
		Duration durataSosta;
		if (a.isBefore(da)) {
			durataSosta = Duration.between(da, LocalTime.of(23, 59)).plusMinutes(1);
		}
		else {
			durataSosta = Duration.between(da, a);
		}
		return costoOrario*durataSosta.toMinutes()/60.0;
	}	
}
