package parkomat.persistence;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

import parkomat.model.FasciaTariffaria;
import parkomat.model.PagamentoAttivo;
import parkomat.model.Tariffa;
import parkomat.model.ItDayOfWeek;

public class MyTariffeReader implements TariffeReader
{
	@Override
	public Map<String,Tariffa> leggiTariffe(Reader reader) throws IOException, BadFileFormatException {
		Map<String,Tariffa> mappaTariffe = new HashMap<>();
		Tariffa t;
		BufferedReader fin = new BufferedReader(reader);
		while ((t = leggiTariffa(fin)) != null) {
			// System.out.println("Letta tariffa " + t.getNomeTariffa());
			mappaTariffe.put(t.getNomeTariffa(), t);
		}	
		return mappaTariffe;
	}
	
	private Tariffa leggiTariffa(BufferedReader reader) throws IOException, BadFileFormatException
	// La tariffa letta NON è valida, in quanto è la pura trascrizione delle fasce a pagamento specificate nel file:
	// mancano ancora tutte quelle gratuite. Prima di usarla occorre quindi invocare makeValid(). 
	{
		String line, nomeTariffa;
		Tariffa tariffa = null;
		try {
			// prima riga
			line = reader.readLine();
			if (line==null) return null;
			if (!line.startsWith("Tariffa")) throw new BadFileFormatException("manca keyword Tariffa");
			else nomeTariffa=(line.split("\\s+"))[1];
			// seconda riga
			line = reader.readLine();
			// creazione tariffa (senza ancora le fasce)
			// argomenti: nome, costo 0.50, franchigia 0, minimo 0, stagione Ottobre-Maggio
			tariffa = makeTariffa(nomeTariffa, line.split(","));
			
			while (!"Fine tariffa".equals(line = reader.readLine())) {
				String[] items = line.split(",");
				// items[0] = "Lunedì"
				// items[1] = "8-13"
				DayOfWeek day = ItDayOfWeek.toDayOfWeek(items[0]);
				//
				String[] ore = items[1].split("-");
				LocalTime oraDa = 	 LocalTime.of(Integer.parseInt(ore[0].trim()),0);
				LocalTime oraFinoA = LocalTime.of(Integer.parseInt(ore[1].trim()),0);
				//
				FasciaTariffaria fascia = new FasciaTariffaria(oraDa, oraFinoA, PagamentoAttivo.A_PAGAMENTO);
				tariffa.addFascia(fascia, day);
			}
		}
		catch (IOException | NoSuchElementException | NumberFormatException e)
		{
			throw new BadFileFormatException(e);
		}
		return tariffa;
	}

	private Tariffa makeTariffa(String name, String[] items) throws BadFileFormatException {
		// items[0] = "costo 0.50"
		// items[1] = "franchigia 60"
		// items[2] = "minimo 60"

		String[] costi = items[0].trim().split("\\s+");
		double costo;
		if (!costi[0].trim().equals("costo")) throw new BadFileFormatException("manca keyword costo, trovato " + costi[0]);
		else costo = Double.parseDouble(costi[1]);
		//
		int franchigia=0, minimo=0;
		// franchigia
		String[] s = items[1].trim().split("\\s+"); // franchigia
		if (s[0].trim().equals("franchigia")) {
			franchigia = Integer.parseInt(s[1].trim());
		}
		// minimo
		s = items[2].trim().split("\\s+"); 	// minimo
		if (s[0].trim().equals("minimo")) {
			minimo = Integer.parseInt(s[1].trim());
		}
		if (!"franchigia minimo".contains(s[0])) {
				throw new BadFileFormatException("keyword sconosciuta: " + s[0]);
		}
		// costruzione tariffa
		return new Tariffa(name, costo, franchigia, minimo);
	}

}
