package parkomat.ui.javafx;

import java.time.LocalDate;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;

import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.DatePicker;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import parkomat.model.Ticket;
import parkomat.ui.controller.Controller;

public class ParkomatPane extends BorderPane {

	private TextArea outputArea;
	private Controller controller;
	private Button bottoneTicket;
	private VBox leftPane, centerPane;
	private ComboBox<String> combo; 
	private DatePicker pickerInizio, pickerFine;
	private TextField fieldInizio, fieldFine;
	private DateTimeFormatter formatter;
	
	public ParkomatPane(Controller controller) {
		this.controller = controller;
		formatter = DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY);
		//
		HBox topPane = new HBox();
		ObservableList<String> nomiTariffe = this.controller.getNomiTariffe();
		combo = new ComboBox<>(nomiTariffe);
		combo.setValue(nomiTariffe.get(0));
		topPane.getChildren().addAll(new Label("Zona tariffaria"), combo);
		this.setTop(topPane);
		//
		leftPane = new VBox();
		leftPane.getChildren().add(new Label(" Inizio sosta: "));
		pickerInizio = new DatePicker(LocalDate.now()); 
		fieldInizio = new TextField();
		fieldInizio.setText(LocalTime.now().format(formatter));
		fieldInizio.setPromptText("Inserire ora di inizio sosta");
		leftPane.getChildren().addAll(pickerInizio,fieldInizio);
		this.setLeft(leftPane);
		//
		centerPane = new VBox();
		centerPane.getChildren().add(new Label(" Fine sosta: "));
		pickerFine = new DatePicker(LocalDate.now()); 
		fieldFine = new TextField();
		fieldFine.setText(LocalTime.now().format(formatter));
		fieldFine.setPromptText("Inserire ora di fine sosta");
		centerPane.getChildren().addAll(pickerFine,fieldFine);
		this.setCenter(centerPane);
		//
		outputArea = new TextArea();
		outputArea.setEditable(false);
		outputArea.setPrefWidth(400);
		outputArea.setFont(Font.font("Courier New", FontWeight.BOLD, 12));
		outputArea.setText("");
		this.setRight(outputArea);
		//
		bottoneTicket = new Button("Emetti ticket");
		bottoneTicket.setOnAction(this::emettiTicket);
		this.setBottom(bottoneTicket);


	}

	private void emettiTicket(ActionEvent event) {
		String nomeTariffa = this.combo.getValue();
		Ticket tk = controller.emitTicket(nomeTariffa, pickerInizio.getValue(), fieldInizio.getText(), pickerFine.getValue(), fieldFine.getText()); 
		this.outputArea.setText(tk!=null ? tk.toString() : "");
	}


}
